@@ -1,9 +1,9 @@
 # -*- coding: utf-8 -*-
 
 """
-Copyright (C) 2010-2012 Wolfgang Rohdewald <wolfgang@rohdewald.de>
+Copyright (C) 2010-2014 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
-kajongg is free software you can redistribute it and/or modify
+Kajongg is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation either version 2 of the License, or
 (at your option) any later version.
@@ -25,11 +25,10 @@ if os.name == 'nt':
     import winsound # pylint: disable=import-error
 
 from common import Debug, Internal
-from util import which, logWarning, m18n, cacheDir, logDebug, \
-    removeIfExists, logException, uniqueList
+from util import which, removeIfExists, uniqueList, elapsedSince
+from log import logWarning, m18n, logDebug, logException
 
-if Internal.haveKDE:
-    from kde import KGlobal, KConfigGroup
+from kde import KGlobal, cacheDir
 
 from tile import Tile
 
@@ -47,22 +46,33 @@ class Sound(object):
     """the sound interface. Use class variables and class methods,
     thusly ensuring no two instances try to speak"""
     enabled = False
-    __hasogg = None
+    __oggName = None
     playProcesses = []
     lastCleaned = None
 
     @staticmethod
     def findOgg():
-        """sets __hasogg to True or False"""
-        if Sound.__hasogg is None:
-            oggName = r'c:\vorbis\oggdec.exe' if os.name == 'nt' else 'ogg123'
-            if not which(oggName):
+        """sets __oggName to exe name or False"""
+        if Sound.__oggName is None:
+            if os.name == 'nt':
+                parentDirectories = KGlobal.dirs().findDirs("appdata", "voices")
+                if parentDirectories:
+                    oggName = os.path.join(parentDirectories[0], 'oggdec.exe')
+                    msg = '' # we bundle oggdec.exe with the kajongg installer, it must be there
+                else:
+                    msg = m18n('No voices will be heard because the program %1 is missing', 'oggdec.exe')
+            else:
+                oggName = 'ogg123'
+                msg = m18n('No voices will be heard because the program %1 is missing', oggName)
+            if which(oggName):
+                Sound.__oggName = oggName
+            else:
+                Sound.__oggName = False
                 Sound.enabled = False
                 # checks again at next reenable
-                logWarning(m18n('No voices will be heard because the program %1 is missing', oggName))
-                return
-            Sound.__hasogg = True
-        return Sound.__hasogg
+                if msg:
+                    logWarning(msg)
+        return Sound.__oggName
 
     @staticmethod
     def cleanProcesses():
@@ -87,7 +97,7 @@ class Sound(object):
                 except OSError:
                     pass
                 if Debug.sound:
-                    game = Internal.field.game
+                    game = Internal.scene.game
                     game.debug('10 seconds passed. Killing %s' % process.name)
             else:
                 remaining.append(process)
@@ -98,28 +108,36 @@ class Sound(object):
         """this is what the user of this module will call."""
         if not Sound.enabled:
             return
-        game = Internal.field.game
+        game = Internal.scene.game
         reactor = Internal.reactor
         if game and not game.autoPlay and Sound.playProcesses:
             # in normal play, wait a moment between two speaks. Otherwise
             # sometimes too many simultaneous speaks make them ununderstandable
             lastSpeakStart = max(x.startTime for x in Sound.playProcesses)
-            if datetime.datetime.now() - lastSpeakStart < datetime.timedelta(seconds=0.3):
+            if elapsedSince(lastSpeakStart) < 0.3:
                 reactor.callLater(1, Sound.speak, what)
                 return
         if os.path.exists(what):
-            if Sound.findOgg():
+            oggName = Sound.findOgg()
+            if oggName:
                 if os.name == 'nt':
+                    # convert to .wav, store .wav in cacheDir
                     name, ext = os.path.splitext(what)
                     assert ext == '.ogg'
-                    wavName = name + '.wav'
+                    nameParts = os.path.normpath(name).split(os.sep)
+                    nameParts = nameParts[nameParts.index('voices')+1:]
+                    wavName = os.path.normpath('{}/{}.wav'.format(cacheDir(), '_'.join(nameParts)))
                     if not os.path.exists(wavName):
-                        args = [r'c:\vorbis\oggdec', '--quiet', what]
-                        process = subprocess.Popen(args)
-                        os.waitpid(process.pid, 0)
-                    winsound.PlaySound(wavName, winsound.SND_FILENAME)
+                        args = [oggName, '-a', '-w', wavName, what]
+                        startupinfo = subprocess.STARTUPINFO()
+                        startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
+                        subprocess.call(args, startupinfo=startupinfo)
+                    try:
+                        winsound.PlaySound(wavName, winsound.SND_FILENAME | winsound.SND_NODEFAULT)
+                    except RuntimeError:
+                        pass
                 else:
-                    args = ['ogg123', '-q', what]
+                    args = [oggName, '-q', what]
                     if Debug.sound:
                         game.debug(' '.join(args))
                     process = subprocess.Popen(args)
@@ -181,9 +199,15 @@ class Voice(object):
         if len(self.directory) == 32:
             if os.path.split(self.directory)[1] == self.md5sum:
                 return 'remote'
-        home = os.environ['HOME'].decode('utf-8')
-        if self.directory.startswith(home):
-            return 'local'
+        if 'HOME' in os.environ:
+            home = os.environ['HOME']
+        elif 'HOMEPATH' in os.environ:
+            home = os.environ['HOMEPATH']
+        else:
+            logException('have neither HOME nor HOMEPATH')
+        if home:
+            if self.directory.startswith(home.decode('utf-8')):
+                return 'local'
         result = os.path.split(self.directory)[0]
         result = os.path.split(result)[1]
         if result == 'voices':
@@ -194,16 +218,15 @@ class Voice(object):
     @staticmethod
     def availableVoices():
         """a list of all voice directories"""
-        if not Voice.__availableVoices and Internal.haveKDE:
+        if not Voice.__availableVoices:
             result = []
             for parentDirectory in KGlobal.dirs().findDirs("appdata", "voices"):
                 parentDirectory = unicode(parentDirectory)
                 for (dirpath, _, _) in os.walk(parentDirectory, followlinks=True):
                     if os.path.exists(os.path.join(dirpath, 's1.ogg')):
                         result.append(Voice(dirpath))
-            config = KGlobal.config()
-            group = KConfigGroup(config, 'Locale')
-            prefLanguages = uniqueList(':'.join(['local', str(group.readEntry('Language')), 'en_uS']).split(':'))
+            group = KGlobal.config().group('Locale')
+            prefLanguages = uniqueList(':'.join(['local', str(group.readEntry('Language')), 'en_US']).split(':'))
             prefLanguages = dict((x[1], x[0]) for x in enumerate(prefLanguages))
             result = sorted(result, key=lambda x: prefLanguages.get(x.language(), 9999))
             if Debug.sound:
@@ -264,6 +287,8 @@ class Voice(object):
 
     def speak(self, text, angle):
         """text must be a sound filename without extension"""
+        if isinstance(text, Tile):
+            text = str(text.exposed)
         fileName = self.localTextName(text, angle)
         if not os.path.exists(fileName):
             if Debug.sound:
